﻿//===============================================================================
// Copyright© 2010 Alachisoft.  All rights reserved.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Collections;

namespace Alachisoft.NCache.Samples.EntityDataModelSample
{
    class EntityDataModelDemo
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main(string[] args)
        {
            try
            {
                // Initialize customer factory
                NorthwindEntities database = new NorthwindEntities();
                Console.WriteLine("EntityDataModel test started");

                IQueryable<Customers> customerQuery = database.Customers.AsQueryable<Customers>();
                IQueryable<Orders> orderQuery = database.Orders.AsQueryable<Orders>();
                
                // - - - New Queries (also mentioend inc onfig)
                string company = "Vaffeljernet";
                customerQuery = from customers in database.Customers where customers.CompanyName == company select customers;
                PrintCustomerDetail(customerQuery);

                DateTime price = DateTime.Now;
                orderQuery = from orders in database.Orders where orders.ShippedDate < price select orders;
                foreach (Orders order in orderQuery)
                {

                    if (order != null)
                    {
                        PrintOrderDetail(order);
                    }

                }
                // - - - 
                
                int choice = 0;
                while (choice != 6)
                {
                    choice = GetUserChoice();

                    string id;
                    switch (choice)
                    {
                        case 1:
                            customerQuery = from allCustomerList in database.Customers
                                            select allCustomerList;
                            PrintCustomerList(customerQuery);
                            break;
                        case 2:
                            id = GetCustomerId();
                            customerQuery = from customerDetail in database.Customers
                                            where customerDetail.CustomerID == id
                                            select customerDetail;
                            PrintCustomerDetail(customerQuery);
                            break;
                        case 3:
                            id = GetCustomerId();
                            orderQuery = from customerOrder in database.Orders
                                         where customerOrder.Customers.CustomerID == id
                                         select customerOrder;
                            PrintCustomerOrders(orderQuery, id);

                            break;
                        case 4:
                            id = GetCustomerId();
                            RemoveCustomer(id, ref database);
                            break;
                        case 5:
                            database.AddToCustomers(AddCustomer());
                            database.SaveChanges();
                            break;
                    }
                }
                Environment.Exit(0);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
        }

        /// <summary>
        /// Get the next user option
        /// </summary>
        /// <returns></returns>
        private static int GetUserChoice()
        {
            Console.WriteLine("");
            Console.WriteLine(" 1- View customers list");
            Console.WriteLine(" 2- View customer details");
            Console.WriteLine(" 3- View customer orders");
            Console.WriteLine(" 4- Delete customer");
            Console.WriteLine(" 5- Add customer");
            Console.WriteLine(" 6- Exit");
            Console.WriteLine("");

            Console.Write("Enter your choice (1 - 6): ");
            try
            {
                int choice = Convert.ToInt32(Console.ReadLine());
                if (choice >= 1 && choice <= 6)
                    return choice;
            }
            catch (Exception)
            {
            }
            Console.WriteLine("Please enter a valid choice (1 - 5)");
            return GetUserChoice();
        }

        /// <summary>
        /// Display customer list
        /// </summary>
        /// <param name="list"></param>
        private static void PrintCustomerList(IQueryable<Customers> list)
        {
            Console.WriteLine("\nCustomer ID    Customer Name");
            Console.WriteLine("-----------    -------------");

            if (list != null)
            {
                foreach (Customers customer in list)
                {
                    Console.WriteLine("{0,-13}  {1,-30}", customer.CustomerID, customer.ContactName);
                }
            }
        }

        /// <summary>
        /// Get user customer id from user
        /// </summary>
        /// <returns></returns>
        private static string GetCustomerId()
        {
            Console.Write("Enter customer ID: ");
            return Console.ReadLine();
        }

        /// <summary>
        /// Display customer details
        /// </summary>
        /// <param name="customer"></param>
        private static void PrintCustomerDetail(IQueryable<Customers> list)
        {
            foreach (Customers customer in list)
            {
                if (customer != null)
                {

                    Console.WriteLine("\nCustomer's Detail");
                    Console.WriteLine("-----------------");

                    Console.WriteLine("Customer ID : " + customer.CustomerID);
                    Console.WriteLine("Name        : " + customer.ContactName);
                    Console.WriteLine("Company     : " + customer.CompanyName);
                    Console.WriteLine("Address     : " + customer.Address);
                }
            }
        }

        /// <summary>
        /// Display customer orders
        /// </summary>
        /// <param name="customer"></param>
        static void PrintCustomerOrders(IEnumerable<Orders> list, string id)
        {
            IEnumerator ie = list.GetEnumerator();
            ie.MoveNext();

            Orders selectedOrder = (Orders)ie.Current;
            if (selectedOrder != null)
            {
                if (selectedOrder.Customers != null)
                {
                    Console.WriteLine("\n" + selectedOrder.Customers.ContactName + "'s Orders");
                }
                else
                {
                    NorthwindEntities database = new NorthwindEntities();

                    IQueryable<Customers> customerQuery = database.Customers.AsQueryable<Customers>();
                    customerQuery = customerQuery.Where(customer => customer.CustomerID == id);

                    IEnumerator enumerator = customerQuery.GetEnumerator();
                    enumerator.MoveNext();

                    Customers cust = (Customers)enumerator.Current;
                    Console.WriteLine("\n" + cust.CustomerID + "'s Orders");
                }
                Console.WriteLine("------------------------");
                Console.WriteLine(" {0,-10} {1,-25} {2,-20}", "Order ID", "Order Date", "Ship Name");
                Console.WriteLine(" {0,-10} {1,-25} {2,-20}", "--------", "-----------", "---------");

                foreach (Orders order in list)
                {

                    if (order != null)
                    {
                        PrintOrderDetail(order);
                    }

                }
            }
        }

        /// <summary>
        /// Display order details
        /// </summary>
        /// <param name="order"></param>
        static void PrintOrderDetail(Orders order)
        {
            if (order != null)
                Console.WriteLine(" {0,-10} {1,-25} {2,-20}", order.OrderID, order.OrderDate.ToString(), order.ShipName);
        }

        /// <summary>
        /// Remove Customers and all tables dependant including Orders and Order details
        /// </summary>
        /// <param name="order"></param>
        static void RemoveCustomer(string id, ref NorthwindEntities database)
        {
            IQueryable<Customers> customerQuery = database.Customers.AsQueryable<Customers>();
            IQueryable<Orders> orderQuery = database.Orders.AsQueryable<Orders>();
            IQueryable<Order_Details> orderDetailQuery = database.Order_Details.AsQueryable<Order_Details>();

            customerQuery = from customertoDelete in database.Customers
                            where customertoDelete.CustomerID == id
                            select customertoDelete;

            orderQuery = from customerOrder in database.Orders
                         where customerOrder.Customers.CustomerID == id
                         select customerOrder;

            foreach (Orders order in orderQuery)
            {

                orderDetailQuery = from detail in database.Order_Details
                                   where detail.Orders.OrderID == order.OrderID
                                   select detail;

                foreach (Order_Details orderDetails in orderDetailQuery)
                {
                    database.AttachTo("Order_Details", orderDetails);
                    database.DeleteObject(orderDetails);

                }
            }
            database.SaveChanges();

            foreach (Orders orders in orderQuery)
            {
                database.AttachTo("Orders", orders);
                database.DeleteObject(orders);
            }
            database.SaveChanges();

            List<Customers> customer = new List<Customers>();
            foreach (Customers c in customerQuery)
                customer.Add(c);
            if (customer.Count() != 0)
            {
                database.AttachTo("Customers", customer[0]);
                database.DeleteObject(customer[0]);
                database.SaveChanges();
            }

        }

        /// <summary>
        /// Fills in Information for new Customer
        /// </summary>
        /// <param name="order"></param>
        static Customers AddCustomer()
        {
            bool validID = false;
            Customers customer = new Customers();

            while (!validID)
            {
                Console.Write("\nEnter Customer ID (maximum length = 5): ");
                customer.CustomerID = Console.ReadLine();
                if (customer.CustomerID.Length > 5 || customer.CustomerID == "")
                {
                    Console.WriteLine("Exception: CustomerID cannot accept string of length > 5");
                }
                else
                    validID = true;
            }

            Console.Write("\nEnter Customer Name: ");
            customer.ContactName = Console.ReadLine();

            Console.Write("\nEnter Customer Company Name: ");
            customer.CompanyName = Console.ReadLine();

            Console.Write("\nEnter Country Name: ");
            customer.Country = Console.ReadLine();

            Console.Write("\nEnter City: ");
            customer.City = Console.ReadLine();

            Console.Write("\nEnter Address: ");
            customer.Address = Console.ReadLine();

            return customer;
        }

    }
}





